// Plant Rosa canina generations data
const plantGenerationsData = [{
    "name": "Earth",
    "parent": "null",
    "fill": "Lime",
    "children": [
        {
            "name": "Asia",
            "parent": "Earth",
            "fill": "orange",
            "children": [
                {
                    "name": "China",
                    "parent": "Asia",
                    "fill": "violet",
                    "children": [
                        {
                            "name": "Beijing",
                            "parent": "China",
                            "fill": "green"
                        },
                        {
                            "name": "Shangai",
                            "parent": "China",
                            "fill": "green"
                        }
                    ]
                },
                {
                    "name": "India",
                    "parent": "Harihara I",
                    "fill": "violet",
                    "children": [
                        {
                            "name": "Mumbai",
                            "parent": "India",
                            "fill": "green"
                        },
                        {
                            "name": "Hyderabad",
                            "parent": "India",
                            "fill": "green"
                        }
                    ]
                }
            ]
        },
        {
            "name": "Europe",
            "parent": "Earth",
            "fill": "orange",
            "children": [
                {
                    "name": "France",
                    "parent": "Europe",
                    "fill": "violet",
                    "children": [
                        {
                            "name": "Paris",
                            "parent": "France",
                            "fill": "green"
                        },
                        {
                            "name": "Lyon",
                            "parent": "France",
                            "fill": "green"
                        }
                    ]
                },
                {
                    "name": "Germany",
                    "parent": "Europe",
                    "fill": "violet",
                    "children": [
                        {
                            "name": "Berlin",
                            "parent": "Germany",
                            "fill": "green"
                        },
                        {
                            "name": "Munich",
                            "parent": "Germany",
                            "fill": "green"
                        }
                    ]
                }
            ]
        }
    ]
}]


// Define the margin, width and height of the visulization.
let margin = {top: 20, right: 120, bottom: 20, left: 120};
let	width = 960 - margin.right - margin.left;
let	height = 500 - margin.top - margin.bottom;
	
var i = 0, duration = 1000, root;

// Variable which holds the whole tree layout
var tree = d3.layout.tree()
	.size([height, width]);

// creates a diagonal generator that connects two points in a tree with custom projection.
var diagonal = d3.svg.diagonal()
	.projection(function(d) { return [d.y, d.x]; });

// Selecting the SVG element as the base to host the tree graph
var svg = d3.select("body").append("svg")
	.attr("width", width + margin.right + margin.left)
	.attr("height", height + margin.top + margin.bottom)
  .append("g")
	.attr("transform", "translate(" + margin.left + "," + margin.top + ")");

// assigning the zeroth element as the root of the Tree and setting the x and y 
root = plantGenerationsData[0];
root.x0 = height / 2;
root.y0 = 0;
  
// Triggering the update method which generates the further tree from root.
update(root);

// Sets the height of the SVG that hosts the Tree structure
d3.select(self.frameElement).style("height", "500px");

function update(source) {

  // Computing the descendant nodes 
  let nodes = tree.nodes(root);
  // Computing the descendant linksg
  let links = tree.links(nodes);

  // Normalize for fixed-depth.
  nodes.forEach(function(d) { d.y = d.depth * 180; });

  // Update the nodes according to the identifiers i.e., the plant generations 
  var node = svg.selectAll("g.node")
	  .data(nodes, function(d) { return d.id || (d.id = ++i); });

  // The below code introduces new nodes at the parent's previous position.
  var nodeEnter = node.enter().append("g")
	  .attr("class", "node")
	  .attr("transform", function(d) { return "translate(" + source.y0 + "," + source.x0 + ")"; });
	
  // The below code creates a circle that runs around the node
  nodeEnter.append("circle")
	  .attr("r", 1e-6)
	  .style("fill", function(d) { return d._children ? "black" : "#fff"; });

  // The below code appends the text to each and every node.
  nodeEnter.append("text")
	  .attr("x", function(d) { return d.children || d._children ? -13 : 13; })
	  .attr("dy", ".35em")
	  .attr("text-anchor", function(d) { return d.children || d._children ? "end" : "start"; })
	  .text(function(d) { return d.name; })
	  .style("fill-opacity", 1e-6);

  // The below code apply transition to the nodes to their new position with in duration of 1000 ms
  var nodeUpdate = node.transition()
	  .duration(duration)
	  .attr("transform", function(d) { return "translate(" + d.y + "," + d.x + ")"; });

  // The below code colors the each node that is set in dataset
  nodeUpdate.select("circle")
	  .attr("r", 10)
    .style("fill", d => d.fill);

  // The below code makes the plant generation name to be fully visible.
  nodeUpdate.select("text")
	  .style("fill-opacity", 1);

  // The below code updates the links between the nodes
  var link = svg.selectAll("path.link")
	  .data(links, function(d) { return d.target.id; });

  // The below code enters new links that connects the current position and the parent's previous position.
  link.enter().insert("path", "g")
	  .attr("class", "link")
	  .attr("d", function(d) {
		var o = {x: source.x0, y: source.y0};
		return diagonal({source: o, target: o});
	  });

  // Transition links to their new position.
  link.transition()
	  .duration(duration)
	  .attr("d", diagonal);
}
